# -*- coding: utf-8 -*-
import os
import ConfigParser


class Config(object):

    def __init__(self, config_file):
        if not os.path.exists(config_file):
            raise Exception('config file not exists')
        self.config_file = config_file
        self.configs = {}
        self.load_config()

    def load_config(self):
        cp = ConfigParser.ConfigParser()
        cp.read(self.config_file)
        for section in cp.sections():
            config = {}
            for key, value in cp.items(section):
                config[key] = value
            self.configs[section] = config

    def get(self, section):
        return self.configs.get(section, None)

    def get_db_params(self, section, original=False):
        config = self.get(section)
        if config and 'host' in config and 'user' in config and 'password' in config and 'db' in config:
            if original:
                return {'host': config['host'], 'user': config['user'], 'passwd': config['password'], 'db': config['db']}
            return {'host': config['host'], 'user': config['user'], 'password': config['password'], 'db': config['db']}
        return None



# -*- coding: utf-8 -*-
import logging
import json
import random

from treq import post
from twisted.internet import defer, reactor, task
from twisted.web import http
from twisted.web.client import HTTPConnectionPool

from wd_crawler_client.lib.md5 import MD5
from wd_crawler_client.lib.queue import DiskQueue, PriorityQueue


class Request(object):

    def __init__(self, task_id, url, headers, data=None, redirect=None, verify=None, is_head=None, return_header=None,
                 encoding=None, timeout=None, retry_times=None, extract_type=None, priority=2, md5_suffix='', **kwargs):
        """
        Request对象
        :param task_id: str，任务id
        :param url: str，待抓取URL
        :param headers: dict，请求头
        :param data: str，请求数据，POST请求有效
        :param redirect: int，是否支持跳转，默认支持(1)
        :param verify: int，是否做https验证，默认支持(1)
        :param is_head: int，是否发送HEAD请求，默认不发送(0)
        :param return_header: int，是否返回响应头，默认不返回(0)
        :param encoding: str，页面编码，默认不指定
        :param timeout: int，超时时间，默认30s
        :param retry_times: int，抓取失败重试，默认3
        :param extract_type: int，解析类型：0->不解析;1->解析百度PC排名结果;2->解析百度移动排名结果;3->解析百度真实URL;
                                            4->解析百度PC URL是否收录;5->解析360PC排名结果;6->解析360移动排名结果;
                                            7->解析搜狗PC排名结果;8->解析搜狗移动排名结果;9->解析网页TDK
        :param priority: int，优先级：2->中;3->高
        :param kwargs: 其他参数
        """
        assert task_id is not None, "%s must have a task_id" % type(self).__name__
        assert url is not None, "%s must have a url" % type(self).__name__
        self.tid = task_id
        self.url = url
        self.uid = MD5.md5(url + md5_suffix)
        self.headers = headers
        self.data = data
        self.redirect = redirect
        self.verify = verify
        self.is_head = is_head
        self.return_header = return_header
        self.encoding = encoding
        self.timeout = timeout
        self.retry_times = retry_times
        self.extract_type = extract_type
        self.priority = priority
        self.__dict__.update(kwargs)

    @property
    def params(self):
        request = {'tid': self.tid, 'uid': self.uid, 'u': self.url, 'hs': self.headers}
        if self.data is not None:
            request['d'] = self.data
        if self.redirect is not None:
            request['r'] = self.redirect
        if self.verify is not None:
            request['v'] = self.verify
        if self.is_head is not None:
            request['ih'] = self.is_head
        if self.return_header is not None:
            request['rh'] = self.return_header
        if self.timeout is not None:
            request['t'] = self.timeout
        if self.retry_times is not None:
            request['rt'] = self.retry_times
        if self.encoding is not None:
            request['e'] = self.encoding
        if self.extract_type is not None:
            request['et'] = self.extract_type
        if self.priority is not None:
            request['pr'] = self.priority
        return json.dumps(request)


class Crawler(object):

    # test
    # server = 'http://127.0.0.1:12000'

    # nginx online
    server = 'http://118.89.92.123:11010'

    def __init__(self, user_id, name, priority=2, db_params=None, user_agents=None):
        assert user_id is not None, "%s must have user_id" % type(self).__user_id__
        self.user_id = user_id
        assert name is not None, "%s must have a name" % type(self).__name__
        self.name = name
        self.task_id = None
        self.priority = priority
        self.pool = HTTPConnectionPool(reactor)
        if db_params:
            from twisted.enterprise import adbapi
            self.dbPool = adbapi.ConnectionPool("MySQLdb", cp_reconnect=True, use_unicode=True, charset='utf8',
                                                init_command='SET names utf8', connect_timeout=30, **db_params)
        if user_agents:
            self.user_agents = user_agents
        else:
            from wd_crawler_client.lib.user_agents import UA_PC
            self.user_agents = UA_PC
        self.queue = PriorityQueue(DiskQueue, [1, 2, 3], path='tmp/%s' % self.name)
        self.requests = {}
        self.max_requests = 300
        logging.basicConfig(filename='logs/%s.log' % self.name, level=logging.INFO,
                            format='%(asctime)s - %(levelname)s - %(name)s : %(message)s')
        self.logger = logging.getLogger('spider')

    @defer.inlineCallbacks
    def start(self):
        success = False
        try:
            response = yield post('%s/connect/' % self.server, data={
                'user_id': self.user_id, 'name': self.name, 'priority': self.priority}, pool=self.pool)
            msg = yield response.text()
            self.log('connect resp: ' + msg, logging.INFO)
            if response.code == http.OK:
                result = yield response.json()
                if result['success']:
                    success = True
                    self.log('connect success, start fetching ...')
                    self.task_id = result['task_id']
                    yield defer.maybeDeferred(self.init_requests)
                    task.LoopingCall(self.fetch).start(2)
                else:
                    self.log('connect failure: ' + result['message'], logging.ERROR)
            else:
                message = yield response.text()
                self.log('connect error: ' + message, logging.ERROR)
        except Exception as e:
            success = False
            import traceback
            traceback.print_exc()
            self.log('connect error: ' + str(e), logging.ERROR)
        finally:
            if not success:
                self.stop()

    def init_requests(self):
        raise NotImplementedError

    def make_request(self, url, headers=None, data=None, redirect=None, verify=None, is_head=None, return_header=None,
                     encoding=None, timeout=None, retry_times=None, extract_type=None, priority=2, md5_suffix='',
                     **kwargs):
        request = Request(self.task_id, url, headers or {'User-Agent': random.choice(self.user_agents)}, data, redirect,
                          verify, is_head, return_header, encoding, timeout, retry_times, extract_type, priority,
                          md5_suffix, **kwargs)
        self.queue.put(request, priority)

    def fetch(self):
        if len(self.requests) == 0 and len(self.queue) == 0:
            self.stop()
        else:
            requests = []
            for _ in range(self.max_requests - len(self.requests)):
                if len(self.queue):
                    request = self.queue.get()
                    requests.append(request)
                    self.requests[request.uid] = request
                else:
                    break
            if requests:
                self.send(requests)
            for _ in range(int(len(self.requests)/100) + 1):
                self.get()

    @defer.inlineCallbacks
    def send(self, requests):
        success = False
        try:
            self.log('send request reqs len:%d' % (len(requests)), logging.INFO)

            response = yield post(
                '%s/task/' % self.server,
                {'task_id': self.task_id, 'urls': json.dumps([request.params for request in requests])}, pool=self.pool)
            if response.code == http.OK:
                success = True
        except Exception as e:
            self.log('send request error: ' + str(e), logging.ERROR)
        finally:
            if not success:
                for request in requests:
                    self.queue.put(request, 3)

    @defer.inlineCallbacks
    def get(self):
        try:
            response = yield post('%s/result/' % self.server, {'task_id': self.task_id}, pool=self.pool)
            if response.code == http.OK:
                response = yield response.json()
                if response['success']:
                    for result in response['results']:
                        try:
                            result = json.loads(result)
                        except:
                            continue
                        if result['uid'] in self.requests:
                            request = self.requests[result['uid']]
                            if 'success' not in result:
                                self.queue.put(request, 3)
                            else:
                                try:
                                    yield defer.maybeDeferred(self.deal, result, request)
                                    self.requests.pop(result['uid'], None)
                                except Exception as e1:
                                    self.log('deal error: ' + str(e1), logging.ERROR)
                                    import traceback
                                    traceback.print_exc()
        except Exception as e:
            self.log('get result error: ' + str(e), logging.ERROR)

    def deal(self, response, request):
        raise NotImplementedError

    def finished(self, success):
        pass

    @defer.inlineCallbacks
    def stop(self, success=True):
        try:
            if self.task_id:
                response = yield post('%s/disconnect/' % self.server, data={'task_id': self.task_id}, pool=self.pool)
                self.log('disconnect resp.code: %d' % (response.code), logging.INFO)
                if response.code == http.OK:
                    result = yield response.json()
                    self.log('disconnect resp:%s' % (result), logging.INFO)
                    if result['success']:
                        self.log('disconnect success, stop fetching ...')
                    else:
                        self.log('disconnect failure: ' + result['message'], logging.ERROR)
                else:
                    message = yield response.text()
                    self.log('disconnect error: ' + message, logging.ERROR)
            yield defer.maybeDeferred(self.finished, success)
        except Exception as e:
            self.log('disconnect error: ' + str(e), logging.ERROR)
        finally:
            reactor.stop()

    def run(self):
        reactor.callWhenRunning(self.start)
        reactor.run()

    def log(self, message, level=logging.DEBUG, **kw):
        self.logger.log(level, message, **kw)

    @staticmethod
    def interaction(cursor, sql, flag='count', defer_warnings=True):
        cursor._cursor._defer_warnings = defer_warnings
        ret = cursor.execute(sql)
        if flag == 'id':
            return cursor.lastrowid
        return cursor.rowcount

    @defer.inlineCallbacks
    def insert(self, table, data):
        """
        将字典直接insert到数据库
        Args:
            table:字符串，插入目标表的名称
            data:字典格式，key为字段名称，value为字段值，如{'id':'1','name':'temp'}
        """
        if len(data) == 0:
            return -1
        try:
            fields, values = zip(*data.items())
            if len(fields) == 0 or len(values) == 0:
                return -1
            sql = "insert into %s(`%s`) values('%s')" % (table, 
                    '`, `'.join(fields),
                    "', '".join(str(value) for value in values))
            ret = yield self.dbPool.runInteraction(self.interaction, sql, 'id')
            return 0
        except Exception as e:
            import traceback
            traceback.print_exc()
            return -1

    @defer.inlineCallbacks
    def insert_many(self, table, datas):
        for data in datas:
            ret = yield self.insert(table, data)
            if ret == -1:
                return ret
        return 0

    @defer.inlineCallbacks
    def update(self, table, data, field):
        """
        将字典直接update到数据库
        Args:
            table:字符串，更新目标表的名称
            data:字典格式，key为字段名称，value为字段值，如{'id':'1','name':'temp'}
            field:唯一索引字段，即根据该字段判断是否为同一条记录，作为where条件
        """
        if len(data) == 0:
            return -1
        try:
            if field not in data:
                return -1
            values = []
            for key, value in data.items():
                values.append("`%s` = '%s'" % (key, str(value)))
            if len(values) == 0:
                return -1
            sql = "update %s set %s where `%s` = '%s'" % (table, ','.join(values), field, str(data[field]))
            yield self.dbPool.runInteraction(self.interaction, sql)
            return 0
        except Exception as e:
            import traceback
            traceback.print_exc()
            return -1

    @defer.inlineCallbacks
    def update_many(self, table, datas, field):
        for data in datas:
            ret = yield self.update(table, data, field)
            if ret == -1:
                return ret
        return 0


class CaptureCrawler(Crawler):

    def __init__(self, name, priority=2, db_params=None, user_agents=None):
        self.server = 'http://10.105.47.191:21010'
        super(CaptureCrawler, self).__init__(name, priority, db_params, user_agents)



# -*- coding: utf8 -*-
import MySQLdb
import time
import traceback


class StoreMysql(object):
    """
    mysql读写相关操作，需安装MySQLdb库
    Args:
        host:数据库ip
        user:数据库用户名
        password:数据库用户密码
        db:数据库名
        port:数据库端口，默认3306
        max_idle_time:数据库连接维持时间
        connect_timeout:连接超时时间
        charset:数据库编码，默认utf8
    """
    def __init__(self, host="", user="", password="", db="", port=3306, max_idle_time=7*3600, connect_timeout=30,
                 reconnect_interval=5, reconnect_times=12, charset="utf8", defer_warnings=False):
        self.host = host
        self.db = db
        self.max_idle_time = float(max_idle_time)
        self.reconnect_interval = reconnect_interval
        self.reconnect_times = reconnect_times
        self.defer_warnings = defer_warnings
        args = dict(host=host, user=user, passwd=password, port=port, db=db,
                    use_unicode=True, charset=charset, init_command='SET names utf8',
                    connect_timeout=connect_timeout)

        self._db = None
        self._db_args = args
        self._last_use_time = time.time()
        try:
            self.reconnect()
        except Exception:
            print("Cannot connect to MySQL on %s" % self.host)

    def __del__(self):
        self.close()

    def close(self):
        if getattr(self, "_db", None) is not None:
            self._db.close()
            self._db = None

    def reconnect(self):
        self.close()
        try:
            self._db = MySQLdb.connect(**self._db_args)
        except Exception:
            time.sleep(self.reconnect_interval)

    def _ensure_connected(self):
        if time.time() - self._last_use_time > self.max_idle_time:
            self.close()
        try_time = 0
        while self._db is None and try_time < self.reconnect_times:
            self.reconnect()
            try_time += 1
        self._last_use_time = time.time()

    def _cursor(self):
        self._ensure_connected()
        cursor = self._db.cursor()
        if self.defer_warnings:
            cursor._defer_warnings = True
        return cursor

    def query(self, sql):
        """
        根据sql查询
        Returns：
            数组的数组，外层数组元素为一行，内存数组元素为一行的一列
        """
        rows = []
        cur = self._cursor()
        try:
            cur.execute(sql)
            self._db.commit()
            rows = cur.fetchall()
        except MySQLdb.OperationalError as e:
            print("Error connecting to MySQL on %s" % self.host)
            traceback.print_exc()
            self.close()
        except Exception as e:
            print("mysql query exception:" + sql)
            traceback.print_exc()
        finally:
            cur.close()
        return rows

    def count(self, tb):
        """
        返回某表的行数
        Args:
            tb:字符串，表名称
        """
        sql = 'select count(*) from %s' %(tb)
        results = self.query(sql)
        if len(results) == 1 and len(results[0]) == 1:
            return int(results[0][0])

    def do(self, sql, flag='lastrowid'):
        """
        执行sql，insert/delete/update操作
        Args:
            sql:要执行的sql
            flag:返回值类型，flag=lastrowid返回lastrowid，flag=rowcount返回rowcount
        """
        cur = self._cursor()
        try:
            cur.execute(sql)
            self._db.commit()
            if flag == 'lastrowid':
                return cur.lastrowid
            elif flag == 'rowcount':
                return cur.rowcount
            return 1
        except MySQLdb.OperationalError as e:
            print("Error connecting to MySQL on %s" % self.host)
            traceback.print_exc()
            self.close()
            return -1
        except Exception as e:
            print("mysql execute exception:" + sql)
            traceback.print_exc()
            return -1
        finally:
            cur.close()

    def save(self, table, data):
        """
        将字典直接insert到数据库
        Args:
            table:字符串，插入目标表的名称
            data:字典格式，key为字段名称，value为字段值，如{'id':'1','name':'temp'}
        """
        if len(data) == 0:
            return -1
        try:
            fields, values = zip(*data.items())
            if len(fields) == 0 or len(values) == 0:
                return -1
            return self.do("insert ignore into %s(`%s`) values('%s')" \
                           % (table, '`, `'.join(fields), "', '".join(MySQLdb.escape_string(str(value)) for value in values)))
        except Exception as e:
            traceback.print_exc()
            return -1

    def update(self, table, data, field):
        """
        将字典直接update到数据库
        Args:
            table:字符串，更新目标表的名称
            data:字典格式，key为字段名称，value为字段值，如{'id':'1','name':'temp'}
            field:唯一索引字段，即根据该字段判断是否为同一条记录，作为where条件
        """
        if len(data) == 0:
            return -1
        try:
            if field not in data:
                return -1
            values = []
            for key, value in data.items():
                values.append("`%s` = '%s'" % (key, MySQLdb.escape_string(str(value))))
            if len(values) == 0:
                return -1
            return self.do("update %s set %s where `%s` = '%s'" % (table, ','.join(values), field, MySQLdb.escape_string(str(data[field]))), flag='rowcount')
        except Exception as e:
            traceback.print_exc()
            return -1

    def save_or_update(self, table, data, field):
        """
        将字典更新到数据库，如果已存在则update，不存在则insert
        Args:
            table:字符串，更新目标表的名称
            data:字典格式，key为字段名称，value为字段值，如{'id':'1','name':'temp'}
            field:唯一索引字段，即根据词字段判断是否为同一条记录，作为where条件
        """
        if len(data) == 0:
            return -1
        try:
            if field not in data:
                return -1
            rows = self.query("select count(1) from %s where `%s` = '%s'" % (table, field, MySQLdb.escape_string(str(data[field]))))
            if rows and rows[0][0] > 0:
                return self.update(table, data, field)
            else:
                return self.save(table, data)
        except Exception as e:
            traceback.print_exc()
            return -1



# -*- coding: utf-8 -*-
# Author: dmhe
# Created on 2017-11-22
import os
import time
import threading
from collections import deque
import cPickle


__all__ = ['HEmpty', 'HQueue']


class HEmpty(Exception):
    pass


class HQueue:

    def __init__(self, path, maxsize=1000, clear=True):
        self.path = path
        if clear:
            self._clear_path()
        self.maxsize = maxsize
        self._init()
        self.mutex = threading.Lock()
        self.not_empty = threading.Condition(self.mutex)
        self.not_full = threading.Condition(self.mutex)
        self.all_tasks_done = threading.Condition(self.mutex)
        self.unfinished_tasks = 0

    def task_done(self):
        self.all_tasks_done.acquire()
        try:
            unfinished = self.unfinished_tasks - 1
            if unfinished <= 0:
                if unfinished < 0:
                    raise ValueError('task_done() called too many times')
                self.all_tasks_done.notify_all()
            self.unfinished_tasks = unfinished
        finally:
            self.all_tasks_done.release()

    def join(self):
        self.all_tasks_done.acquire()
        try:
            while self.unfinished_tasks:
                self.all_tasks_done.wait()
        finally:
            self.all_tasks_done.release()

    def qsize(self):
        self.mutex.acquire()
        n = self._qsize
        self.mutex.release()
        return n

    def empty(self):
        self.mutex.acquire()
        n = not self._qsize
        self.mutex.release()
        return n

    def put(self, item):
        self.not_full.acquire()
        try:
            self._put(item)
            self.unfinished_tasks += 1
            self.not_empty.notify()
        finally:
            self.not_full.release()

    def get(self, block=True, timeout=None):
        self.not_empty.acquire()
        try:
            if not block:
                if not self._qsize:
                    raise HEmpty
            elif timeout is None:
                while not self._qsize:
                    self.not_empty.wait()
            elif timeout < 0:
                raise ValueError("'timeout' must be a non-negative number")
            else:
                end_time = time.time() + timeout
                while not self._qsize:
                    remaining = end_time - time.time()
                    if remaining <= 0.0:
                        raise HEmpty
                    self.not_empty.wait(remaining)
            return self._get()
        finally:
            self.not_empty.release()

    def get_nowait(self):
        return self.get(False)

    def _init(self):
        self.queue = deque()
        self._qsize = 0
        self.tmp_queue = deque()
        self.file_queue = deque()
        self.file_num = 1
        if not os.path.exists(self.path):
            os.makedirs(self.path)

    def _put(self, item):
        if len(self.tmp_queue) == 0:
            if len(self.file_queue) == 0:
                if len(self.queue) < self.maxsize:
                    self.queue.append(item)
                else:
                    self.tmp_queue.append(item)
            else:
                self.tmp_queue.append(item)
        else:
            self.tmp_queue.append(item)
            if len(self.tmp_queue) == self.maxsize:
                self._write()
        self._qsize += 1

    def _write(self):
        file_name = 'q%d.cpk' % self.file_num
        with open(os.path.join(self.path, file_name), 'ab+') as f:
            cPickle.dump(self.tmp_queue, f, cPickle.HIGHEST_PROTOCOL)
            self.file_queue.append(file_name)
            self.tmp_queue.clear()
            self.file_num += 1

    def _get(self):
        if len(self.queue) == 0:
            if len(self.file_queue) > 0:
                self._read()
            else:
                self.queue, self.tmp_queue = self.tmp_queue, self.queue
        self._qsize -= 1
        return self.queue.popleft()

    def _read(self):
        file_name = self.file_queue.popleft()
        file_path = os.path.join(self.path, file_name)
        with open(file_path, 'rb') as f:
            self.queue = cPickle.load(f)
        os.remove(file_path)

    def _clear_path(self):
        if os.path.exists(self.path):
            for f in os.listdir(self.path):
                os.remove(os.path.join(self.path, f))
            os.rmdir(self.path)

    def __del__(self):
        self._clear_path()



# -*- coding: utf-8 -*-
# Author: dmhe
# Created on 2018-12-07
import pickle
import os
from collections import deque


__all__ = ['SimpleQueue', 'DiskQueue', 'PriorityQueue']


class SimpleQueue(object):

    def __init__(self):
        self.queue = deque()
        self.put = self.queue.append

    def get(self):
        queue = self.queue
        return queue.popleft() if queue else None

    def __len__(self):
        return len(self.queue)


class DiskQueue:

    def __init__(self, path, maxsize=1000, clear=True):
        self.path = path
        if clear:
            self._clear_path()
        if not os.path.exists(self.path):
            os.makedirs(self.path)
        self.maxsize = maxsize
        self.queue = deque()
        self.tmp_queue = deque()
        self.file_queue = deque()
        self.file_num = 1
        self._qsize = 0

    def put(self, item):
        if len(self.tmp_queue) == 0:
            if len(self.file_queue) == 0:
                if len(self.queue) < self.maxsize:
                    self.queue.append(item)
                else:
                    self.tmp_queue.append(item)
            else:
                self.tmp_queue.append(item)
        else:
            self.tmp_queue.append(item)
            if len(self.tmp_queue) == self.maxsize:
                self._write()
        self._qsize += 1

    def get(self):
        if not self._qsize:
            return None
        if len(self.queue) == 0:
            if len(self.file_queue) > 0:
                self._read()
            else:
                self.queue, self.tmp_queue = self.tmp_queue, self.queue
        self._qsize -= 1
        return self.queue.popleft()

    def _write(self):
        file_name = 'q%d.cpk' % self.file_num
        with open(os.path.join(self.path, file_name), 'ab+') as f:
            pickle.dump(self.tmp_queue, f, pickle.HIGHEST_PROTOCOL)
            self.file_queue.append(file_name)
            self.tmp_queue.clear()
            self.file_num += 1

    def _read(self):
        file_name = self.file_queue.popleft()
        file_path = os.path.join(self.path, file_name)
        with open(file_path, 'rb') as f:
            self.queue = pickle.load(f)
        os.remove(file_path)

    def _clear_path(self):
        if os.path.exists(self.path):
            for f in os.listdir(self.path):
                os.remove(os.path.join(self.path, f))
            os.rmdir(self.path)

    def __len__(self):
        return self._qsize

    def __del__(self):
        self._clear_path()


class PriorityQueue(object):

    def __init__(self, queue_class, priorities, **kwargs):
        self.queues = {}
        self.priorities = sorted(priorities, reverse=True)
        for priority in self.priorities:
            if queue_class == DiskQueue:
                path = kwargs.get('path', None)
                if path:
                    path = os.path.join(path, str(priority))
                    self.queues[priority] = queue_class(path, kwargs.get('maxsize', 1000), kwargs.get('clear', True))
                else:
                    raise Exception('param path unset')

    def put(self, item, priority):
        if priority in self.queues:
            self.queues[priority].put(item)

    def get(self):
        for priority in self.priorities:
            if len(self.queues[priority]):
                return self.queues[priority].get()
        return None

    def __len__(self):
        return sum(len(x) for x in self.queues.values()) if self.queues else 0



#!/usr/bin/env python

try:
    # from setuptools import setup
    from setuptools import find_packages, setup, Command
    import io
    import os
    import sys
    from shutil import rmtree
except ImportError:
    from distutils.core import setup


# Package meta-data.
NAME = 'wd-crawler-client'
DESCRIPTION = 'spider framework for winndoo.'
URL = 'http://git.winndoo.cn:82/python/download_center/downloader_client.git'
EMAIL = 'baozhu.zhang@winndoo.com'
AUTHOR = 'mingwei.jie baozhu.zhang'
REQUIRES_PYTHON = '>=3.6.0'
VERSION = '6.0.2'


here = os.path.abspath(os.path.dirname(__file__))

# Import the README and use it as the long-description.
# Note: this will only work if 'README.md' is present in your MANIFEST.in file!
try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

# Load the package's __version__.py module as a dictionary.
about = {}
if not VERSION:
    project_slug = NAME.lower().replace("-", "_").replace(" ", "_")
    with open(os.path.join(here, project_slug, '__version__.py')) as f:
        exec(f.read(), about)
else:
    about['__version__'] = VERSION



class UploadCommand(Command):
    """Support setup.py upload."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        self.status('Building Source and Wheel (universal) distribution…')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPI via Twine…')
        os.system('twine upload dist/*')

        self.status('Pushing git tags…')
        os.system('git tag v{0}'.format(about['__version__']))
        os.system('git push --tags')

        sys.exit()


setup(
      name=NAME,
      version=about['__version__'],
      description=DESCRIPTION,
      long_description=long_description,
      long_description_content_type='text/markdown',
      author=AUTHOR,
      author_email=EMAIL,
      python_requires=REQUIRES_PYTHON,
      url=URL,
      keywords=['crawler', 'wendao'],
      packages=['wd_crawler_client', 'wd_crawler_client.crawler', 'wd_crawler_client.lib'],
      install_requires=[
          'mysqlclient>=1.4.6',
          'Twisted>=18.9.0',
          'treq>=18.6.0',
      ],
      # $ setup.py publish support.
      cmdclass={
          'upload': UploadCommand,
      },
)




# -*- coding: utf8 -*-
import hashlib


class MD5(object):

    @staticmethod
    def md5(s):
        if isinstance(s, str):
            s = s.encode('utf-8')
        m = hashlib.md5()
        m.update(s)
        return m.hexdigest()



# -*- coding: utf-8 -*-
import smtplib
import traceback
from email.header import Header
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText


class Mail(object):

    def __init__(self, host, port, user, password):
        self.user = user
        self.server = smtplib.SMTP_SSL(host, port)
        self.server.login(user, password)

    def send(self, to_users, subject, content, attach_name=None, attach_content=None):
        msg = MIMEMultipart()
        msg['From'] = Header(self.user, 'utf-8').encode()
        msg['To'] = Header(u';'.join(to_users), 'utf-8').encode()
        msg['Subject'] = Header(subject, 'utf-8').encode()
        msg.attach(MIMEText(content, 'plain', 'utf-8'))
        if attach_name and attach_content:
            attach = MIMEText(attach_content, 'base64', 'utf-8')
            attach["Content-Type"] = 'application/octet-stream'
            attach["Content-Disposition"] = 'attachment; filename="%s"' % attach_name
            msg.attach(attach)
        try:
            self.server.sendmail(self.user, to_users, msg.as_string())
        except Exception as e:
            print(e)
            traceback.print_exc()

    def __del__(self):
        self.server.quit()
